@@ -1,6 +1,6 @@
 /*
  * This file is part of the HDRL
- * Copyright (C) 2013 European Southern Observatory
+ * Copyright (C) 2013,2014 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,12 +39,6 @@
                             Static Prototypes
  -----------------------------------------------------------------------------*/
 
-/* TMP */
-static cpl_error_code hdrl_image_dump_structure(const hdrl_image *, FILE *) ;
-static cpl_error_code hdrl_image_dump_window(const hdrl_image *,
-        cpl_size, cpl_size, cpl_size, cpl_size, FILE *) ;
-/* END TMP */
-
 /*----------------------------------------------------------------------------*/
 /**
  * @addtogroup hdrl_imagelist
@@ -56,6 +50,22 @@ static cpl_error_code hdrl_image_dump_window(const hdrl_image *,
                             Function codes
  -----------------------------------------------------------------------------*/
 
+/* ---------------------------------------------------------------------------*/
+/**
+ * @internal
+ * @brief resize buffer to set size
+ * @param h    imagelist to update
+ * @param size capacity the buffer should have, minimum 128
+ *
+ * only changes the buffer size, not the number of contained images
+ */
+/* ---------------------------------------------------------------------------*/
+static void hdrl_imagelist_set_capacity(hdrl_imagelist * h, cpl_size size)
+{
+    h->capacity = CX_MAX(h->ni, CX_MAX(128, size));
+    h->images = cpl_realloc(h->images, h->capacity * sizeof(h->images[0]));
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Create an empty imagelist
@@ -67,6 +77,7 @@ static cpl_error_code hdrl_image_dump_window(const hdrl_image *,
 hdrl_imagelist * hdrl_imagelist_new(void)
 {
     hdrl_imagelist * h = cpl_calloc(1, sizeof(hdrl_imagelist));
+    hdrl_imagelist_set_capacity(h, 128);
     return h;
 }
 
@@ -74,7 +85,7 @@ hdrl_imagelist * hdrl_imagelist_new(void)
 /**
   @brief    Create an hdrl_imagelist out of 2 cpl_imagelist 
   @param    imlist    the list of image
-  @param    errlist   the list of errorѕ
+  @param    errlist   the list of errors
   @return   The new hdrl_imagelist
 
   Possible cpl_error_code set in this function:
@@ -85,10 +96,36 @@ hdrl_imagelist * hdrl_imagelist_create(
         cpl_imagelist   *   imlist,
         cpl_imagelist   *   errlist)
 {
-    cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    hdrl_imagelist  *   himlist ;
+    cpl_image       *   error ;
     
-    // TODO
-    return hdrl_imagelist_new() ;
+    /* Check Entries */
+    cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    if (errlist) {
+        cpl_ensure(cpl_imagelist_get_size(imlist) == 
+                cpl_imagelist_get_size(errlist), CPL_ERROR_ILLEGAL_INPUT, NULL);
+    }
+    
+    /* Create the new HDRL image list */
+    himlist = hdrl_imagelist_new() ;
+    
+    /* Loop on the input image list */
+    for (cpl_size i = 0; i < cpl_imagelist_get_size(imlist); i++) {
+        /* Get the error image */
+        hdrl_image * tmp ;
+        if (errlist) {
+            error = cpl_imagelist_get(errlist, i) ;
+        } else {
+            error = NULL ;
+        }
+        /* Create the HDRL image */
+        tmp = hdrl_image_create(cpl_imagelist_get_const(imlist, i), error) ;
+
+        /* Fill the HDRL image list */
+        hdrl_imagelist_set(himlist, tmp, i);
+
+    }
+    return himlist ;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -108,6 +145,46 @@ cpl_size hdrl_imagelist_get_size(const hdrl_imagelist * himlist)
     return himlist->ni;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Get number of colums of  images in the imagelist
+  @param    himlist     the list of images
+  @return   The number of columns of images or -1 on error
+
+  assumes the imagelist is uniform (cpl_imagelist_is_uniform)
+
+  Possible cpl_error_code set in this function:
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_ILLEGAL_INPUT if the list is empty
+ */
+/*----------------------------------------------------------------------------*/
+cpl_size hdrl_imagelist_get_size_x(const hdrl_imagelist * himlist)
+{
+    cpl_ensure(himlist != NULL, CPL_ERROR_NULL_INPUT, -1);
+    cpl_ensure(himlist->ni > 0, CPL_ERROR_ILLEGAL_INPUT, -1);
+    return hdrl_image_get_size_x(hdrl_imagelist_get_const(himlist, 0));
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Get number of rows of  images in the imagelist
+  @param    himlist     the list of images
+  @return   The number of rows of images or -1 on error
+
+  assumes the imagelist is uniform (cpl_imagelist_is_uniform)
+
+  Possible cpl_error_code set in this function:
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_ILLEGAL_INPUT if the list is empty
+ */
+/*----------------------------------------------------------------------------*/
+cpl_size hdrl_imagelist_get_size_y(const hdrl_imagelist * himlist)
+{
+    cpl_ensure(himlist != NULL, CPL_ERROR_NULL_INPUT, -1);
+    cpl_ensure(himlist->ni > 0, CPL_ERROR_ILLEGAL_INPUT, -1);
+    return hdrl_image_get_size_y(hdrl_imagelist_get_const(himlist, 0));
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Get an image from a list of images
@@ -216,9 +293,11 @@ cpl_error_code hdrl_imagelist_set(
     }
 
     if (pos == himlist->ni) {
+        /* double buffer if required */
+        if (pos >= himlist->capacity) {
+            hdrl_imagelist_set_capacity(himlist, 2 * pos);
+        }
         himlist->ni++;
-        himlist->images = cpl_realloc(himlist->images,
-                                     (size_t)himlist->ni * sizeof(hdrl_image*));
     } else {
         /* Check if the image at the position to be overwritten
            is present in only one position */
@@ -280,6 +359,11 @@ hdrl_image * hdrl_imagelist_unset(
     /* Decrement of the size */
     himlist->ni--;
 
+    /* shrink the buffer if its significantly too large */
+    if (himlist->ni < himlist->capacity / 2) {
+        hdrl_imagelist_set_capacity(himlist, himlist->ni / 2);
+    }
+
     return out;
 }
 
@@ -497,8 +581,10 @@ void hdrl_imagelist_unwrap(hdrl_imagelist * himlist)
 typedef struct {
     const hdrl_imagelist * hlist;
     cpl_size ny;
+    cpl_size prev_pos;
     cpl_size pos;
     cpl_size nrows;
+    cpl_size overlap;
     hdrl_imagelist * last_view;
 } hdrl_imagelist_row_slices_iter;
 
@@ -508,29 +594,56 @@ static void * hdrl_imagelist_row_slices_next(hdrl_iter * it)
     if (s->pos > s->ny) {
         return NULL;
     }
-    cpl_size upper = CX_MIN(s->pos + s->nrows - 1, s->ny);
-    /* TODO add const iterator?  could set memory readonly */
-    hdrl_imagelist * view = hdrl_imagelist_row_view((hdrl_imagelist*)s->hlist,
-                                                    s->pos, upper);
-    s->pos = upper + 1;
-    if (hdrl_iter_check(it, HDRL_ITER_OWN_OUTPUT)) {
+    cpl_size lower = CX_MAX(s->pos - s->overlap, 1);
+    cpl_size upper = CX_MIN(s->pos + s->nrows + s->overlap - 1, s->ny);
+    /* const iterator means you cannot modify the data but the created views
+     * can have a NULL bpm which can be faster e.g. when calling
+     * cpl_image_new_from_accepted on the view */
+    /* TODO could set memory readonly? */
+    hdrl_imagelist * view;
+    if (hdrl_iter_check(it, HDRL_ITER_CONST)) {
+        view = (hdrl_imagelist*)
+            hdrl_imagelist_const_row_view((hdrl_imagelist*)s->hlist,
+                                          lower, upper);
+    }
+    else {
+        view = hdrl_imagelist_row_view((hdrl_imagelist*)s->hlist,
+                                       lower, upper);
+    }
+
+    s->prev_pos = s->pos;
+    s->pos = CX_MIN(s->pos + s->nrows - 1, s->ny) + 1;
+    if (hdrl_iter_check(it, HDRL_ITER_OWNS_DATA)) {
         hdrl_imagelist_delete(s->last_view);
         s->last_view = view;
     }
     return view;
 }
 
+hdrl_il_rowsliceiter_data
+hdrl_imagelist_iter_row_slices_get_data(const hdrl_iter * it)
+{
+    hdrl_imagelist_row_slices_iter * s = hdrl_iter_state((hdrl_iter*)it);
+
+    if (s->prev_pos == 1) {
+        return (hdrl_il_rowsliceiter_data){1., CX_MIN(s->nrows, s->ny)};
+    }
+    else {
+        return (hdrl_il_rowsliceiter_data){s->overlap + 1, s->overlap + s->pos - s->prev_pos};
+    }
+}
+
 static cpl_size hdrl_imagelist_row_slices_length(hdrl_iter * it)
 {
     hdrl_imagelist_row_slices_iter * s = hdrl_iter_state(it);
     return s->ny / s->nrows + ((s->ny % s->nrows) != 0);
 }
 
-static void hdrl_imagelist_iter_delete(void * state)
+static void hdrl_imagelist_iter_delete(void * it)
 {
-    if (!state)
+    if (!it)
         return;
-    hdrl_imagelist_row_slices_iter * s = state;
+    hdrl_imagelist_row_slices_iter * s = hdrl_iter_state(it);
     hdrl_imagelist_delete(s->last_view);
     cpl_free(s);
 }
@@ -538,7 +651,8 @@ static void hdrl_imagelist_iter_delete(void * state)
 /* TODO, add offset and stride for parallel processing? or imagelist_split? */
 hdrl_iter * hdrl_imagelist_get_iter_row_slices(const hdrl_imagelist * hlist,
                                                  cpl_size nrows,
-                                                 hdrl_iter_type_t flags)
+                                                 cpl_size overlap,
+                                                 hdrl_iter_flags flags)
 {
     cpl_ensure(hlist, CPL_ERROR_NULL_INPUT, NULL);
     /* 0 accepted for now, could mean choosen by function */
@@ -548,8 +662,10 @@ hdrl_iter * hdrl_imagelist_get_iter_row_slices(const hdrl_imagelist * hlist,
 
     hdrl_imagelist_row_slices_iter * state = cpl_malloc(sizeof(*state));
     state->hlist = hlist;
-    state->ny = hdrl_image_get_size_y(hdrl_imagelist_get_const(hlist, 0));
+    state->ny = hdrl_imagelist_get_size_y(hlist);
+    state->prev_pos = 1;
     state->pos = 1;
+    state->overlap = CX_MAX(overlap, 0);
     state->nrows = CX_MAX(nrows, 1);
     state->last_view = NULL;
 
@@ -561,23 +677,4 @@ hdrl_iter * hdrl_imagelist_get_iter_row_slices(const hdrl_imagelist * hlist,
 }
 
 
-/* TMP */
-static cpl_error_code hdrl_image_dump_structure(
-        const hdrl_image    *   himg,
-        FILE                *   stream)
-{
-    return CPL_ERROR_NONE;
-}
-static cpl_error_code hdrl_image_dump_window(
-        const hdrl_image    *   himg,
-        cpl_size                llx,
-        cpl_size                lly,
-        cpl_size                urx,
-        cpl_size                ury,
-        FILE *                  stream)
-{
-    return CPL_ERROR_NONE;
-}
-/* END TMP */
-
 /**@}*/
