@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 try:
   import numpy
   from pipeline_display import *
@@ -26,7 +27,7 @@ class PlotableReducedArc :
                                    self.arc.crpix1)
     self.arcdisp.display(subplot, title, tooltip, self.arc.image)
     
-class PlotableFlat(object) :
+class PlotableSpecFlat(object) :
   def __init__(self, fits_flat, fits_slittrace):
     self.flat      = PipelineProduct(fits_flat)
     self.slittrace = None
@@ -42,7 +43,7 @@ class PlotableFlat(object) :
     #Reading the polinomial traces
     if self.slittrace is not None:
       ndegree = self.slittrace.getTableNcols(1) - 1
-      self.nslits  = self.slittrace.getTableNrows(1) / 2
+      self.nslits  = self.slittrace.getTableNrows(1) // 2
       degreecols = []
       for deg in range(ndegree):
         colname = 'c%d'%deg
@@ -91,18 +92,18 @@ class PlotableFlat(object) :
         subplot.plot(self.xpos_traces[slit], self.ypos_bottom_traces[slit],
                      linestyle='solid',color='darkred')
 
-class PlotableNormFlat (PlotableFlat) :
+class PlotableNormSpecFlat (PlotableSpecFlat) :
   def __init__(self, fits_flat, fits_slittrace):
-    super(PlotableNormFlat, self).__init__(fits_flat, fits_slittrace)
+    super(PlotableNormSpecFlat, self).__init__(fits_flat, fits_slittrace)
 
   def plot(self, subplot, title, tooltip):
     self.flatdisp.setZLimits((0.9, 1.1))
     self.flat.image[self.flat.image > 5.] = 0
-    super(PlotableNormFlat, self).plot(subplot, title, tooltip)
+    super(PlotableNormSpecFlat, self).plot(subplot, title, tooltip)
 
-class PlotableRawFlat (PlotableFlat) :
+class PlotableRawSpecFlat (PlotableSpecFlat) :
   def __init__(self, fits_flat_raw, fits_master_flat, fits_slittrace):
-    super(PlotableRawFlat, self).__init__(fits_flat_raw, fits_slittrace)
+    super(PlotableRawSpecFlat, self).__init__(fits_flat_raw, fits_slittrace)
     if self.slittrace is not None and fits_master_flat is not None :
       master_flat = PipelineProduct(fits_master_flat)
       self.trimm_lly = master_flat.all_hdu[0].header.get('HIERARCH ESO QC TRIMM LLY')
@@ -117,7 +118,7 @@ class PlotableRawFlat (PlotableFlat) :
     
 
   def plot(self, subplot, title, tooltip):
-    super(PlotableRawFlat, self).plot(subplot, title, tooltip)
+    super(PlotableRawSpecFlat, self).plot(subplot, title, tooltip)
 
 class PlotableSpatialMap :
   def __init__(self, fits_spatialmap):
